/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMElement;

[scriptable, uuid(AA9DEF4E-2E59-412d-A6DF-B76F52167795)]
interface nsIListBoxObject : nsISupports
{
  long getRowCount();
  long getNumberOfVisibleRows();
  long getIndexOfFirstVisibleRow();

  void ensureIndexIsVisible(in long rowIndex);
  void scrollToIndex(in long rowIndex);
  void scrollByLines(in long numLines);

  nsIDOMElement getItemAtIndex(in long index);
  long getIndexOfItem(in nsIDOMElement item);
};

%{C++
class nsIBoxObject;

nsresult
NS_NewListBoxObject(nsIBoxObject** aResult);

%}
